@@ -1,7 +1,7 @@
 package Git::CPAN::Patch::Command::Clone;
 our $AUTHORITY = 'cpan:YANICK';
 #ABSTRACT: Clone a CPAN module's history into a new git repository
-$Git::CPAN::Patch::Command::Clone::VERSION = '2.3.3';
+$Git::CPAN::Patch::Command::Clone::VERSION = '2.3.4';
 use 5.20.0;
 
 use strict;
@@ -28,7 +28,7 @@ has _seen_imports => (
 );
 
 
-before [ qw/import_release clone_git_repo /] => sub($self,$release) {
+before [ qw/import_release clone_git_repo /] => sub($self,$release,@) {
     return if $self->_seen_imports;
     $self->_set_seen_imports(1);
 
@@ -41,7 +41,7 @@ before [ qw/import_release clone_git_repo /] => sub($self,$release) {
     $self->set_root($target);
 };
 
-after [ qw/ clone_git_repo import_release /] => sub($self) {
+after [ qw/ clone_git_repo import_release /] => sub($self,@) {
     $self->git_run( 'reset', '--hard', $self->last_commit );
 };
 
@@ -61,7 +61,7 @@ Git::CPAN::Patch::Command::Clone - Clone a CPAN module's history into a new git
 
 =head1 VERSION
 
-version 2.3.3
+version 2.3.4
 
 =head1 SYNOPSIS
 
